package com.elab.core.dao;/**
 * Created by liukx on 2018/1/4.
 */

import com.elab.core.bean.PageModel;
import org.springframework.jdbc.core.RowMapper;

import java.util.List;
import java.util.Map;

/**
 * 单表公共操作处理
 *
 * @author Liukx
 * @create 2018-01-04 11:18
 * @email liukx@elab-plus.com
 **/
public interface IBaseDaoSupport<T> {

    /**
     * 插入一个对象
     *
     * @param obj 对象内容 必须是构造参数
     * @return
     * @throws Exception
     */
    public int insert(T obj) throws Exception;

    /**
     * 根据主键修改内容
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public int updateById(T obj) throws Exception;

    /**
     * 删除一个对象
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public int delete(T obj) throws Exception;

    /**
     * 查询一个对象
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public T selectByObject(T obj) throws Exception;

    /**
     * 根据主键查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    public T selectById(String id) throws Exception;

    /**
     * 查询一个集合,参数可以是任意的
     *
     * @param obj
     * @return
     * @throws Exception
     */
    public List<T> selectByList(T obj) throws Exception;

    /**
     * 查询一个分页集合
     *
     * @param pageModel 分页参数
     * @param o         查询字段
     * @return
     * @throws Exception
     */
    public PageModel<T> selectByPage(PageModel pageModel, T o) throws Exception;


    /**
     * 查询一个Mapper对象
     * 功能描述: 查询一个由RowMapper封装的返回结果对象
     * 不调用反射去构建一个Bean对象,把set赋值操作交给用户
     *
     * @param obj
     * @param rowMapper
     * @return
     * @throws Exception
     */
    public List<T> selectByMapper(T obj, RowMapper<T> rowMapper) throws Exception;

    /**
     * 批量添加
     *
     * @param list
     * @return
     */
    public int[] batchInsertByObject(List<T> list);

    /**
     * 批量添加
     *
     * @param list
     * @return
     */
    public int[] batchInsertByMap(List<Map> list);

    /**
     * 批量添加
     *
     * @param list
     * @return
     */
    public int[] batchUpdateByObject(List<T> list);

    /**
     * 批量添加
     *
     * @param list
     * @return
     */
    public int[] batchUpdateByMap(List<Map> list);

}
